"use strict";
(function($) {

    window.DmCommentModule = function(wrapper, provider, localizer, dmClient) {
        this.constructor.call(this, wrapper, provider, localizer, dmClient);
    }
    $.extend(DmCommentModule.prototype, {

        constructor: function(wrapper, provider, localizer, dmClient) {
            this.provider = provider;
            this.wrapper = wrapper;
            this.client = dmClient;
            this.localizer = localizer;
            this.initInterface();
        },

        initInterface: function() {
            this.ratingModule = new DmRatingModule(this.provider.id, null, 'dm-rating-wrapper');
            this.initForm();
            this.initList();
        },

        comment: function(e) {
            var username = $(e.target).parent().find('.dm-comment-username').val();
            var cardnumber = $(e.target).parent().find('.dm-comment-cardnumber').val();
            var text = $(e.target).parent().find('.dm-comment-text').val();
            this.wrapper.find("div[data-error]").text('');

            var valid = true;
            if (username == '') {
                this.wrapper.find("div[data-error='authentication']").html(this.localizer.message('username_cannot_be_empty') + '<br />');
                valid = false;
            }
            if (cardnumber == '') {
                this.wrapper.find("div[data-error='authentication']").append(this.localizer.message('cardnumber_cannot_be_empty'));
                valid = false;
            }
            if (text == '') {
                this.wrapper.find("div[data-error='text']").text(this.localizer.message('text_cannot_be_empty'));
                valid = false;
            }
            if (!valid) {
                return false;
            }
            this.client.comment(
                this.provider.id,
                username,
                cardnumber,
                text,
                this.ratingModule.getValue(),
                function (d) {
                    this.wrapper.find('.dm-comment-error').text('');
                    this.wrapper.find('input.dm-comment-username').val('');
                    this.wrapper.find('input.dm-comment-cardnumber').val('');
                    this.wrapper.find('textarea.dm-comment-text').val('');
                    this.pagination.page = 1;
                    this.getComments();
                }.bind(this),
                function () {
                    this.wrapper.find("div[data-error='authentication']").text(this.localizer.message('user_authentication_failed'));
                }.bind(this)
            );
        },

        initForm: function() {
            var form = $('<div />').
                    addClass('dm-comments-form-wrapper').
                    appendTo(this.wrapper);

            $('<h2 />').
                    addClass('dm-header').
                    text(this.localizer.message('comment_share_your_experience')).
                    appendTo(form);

            if (!JSON.parse(discount_search['is_user_logged_in'])) {
                $('<div data-error="authentication" />').
                        addClass('dm-comment-error').
                        appendTo(form);
                $('<div />').
                    addClass('dm-comment-username-wrapper').
                    append(
                        $('<input />').
                            addClass('dm-comment-username').
                            attr('placeholder', this.localizer.message('comment_username') + '*')
                    ).
                    append(
                        $('<span />').
                            addClass('fa').
                            addClass('fa-user')
                    ).
                    appendTo(form);

                    $('<div />').
                        addClass('dm-comment-cardnumber-wrapper').
                        append(
                            $('<input />').
                                addClass('dm-comment-cardnumber').
                                attr('placeholder', this.localizer.message('comment_cardnumber') + '*')
                        ).
                        append(
                            $('<span />').
                                addClass('fa').
                                addClass('fa-credit-card')
                        ).
                        appendTo(form);

                $('<div />').
                        addClass('dm-comment-error').
                        appendTo(form);
            }

            $('<div data-error="text" />').
                        addClass('dm-comment-error').
                        appendTo(form);
            $('<textarea />').
                    addClass('dm-comment-text').
                    attr('placeholder', this.localizer.message('comment_your_comment') + '...').
                    appendTo(form);

            $('<strong />').
                    addClass('dm-rating-header').
                    text(this.localizer.message('comment_rating') + ':').
                    append(this.ratingModule.render()).
                    appendTo(form);

            $('<button />').
                    addClass('dm-btn').
                    addClass('dm-btn-active').
                    text(this.localizer.message('comment_send')).
                    on(
                        'click',
                        this.comment.bind(this)
                    ).
                    appendTo(form);

            $('<div />').
                    addClass('clearer').
                    appendTo(form);
        },

        initList: function() {
            var list = $('<div />').
                    addClass('dm-comments-list-wrapper').
                    appendTo(this.wrapper);

            $('<h2 />').
                    addClass('dm-header').
                    text(this.localizer.message('comment_customers_comments')).
                    appendTo(list);

            this.commentList = $('<div />').
                    addClass('dm-comments-list').
                    appendTo(list);

            var paginationDiv = $('<div />').
                    attr('data-dm-target', 'pagination-pages').
                    addClass('dm-comments-pagination').
                    appendTo(list);

            $('<div />').
                    addClass('clearer').
                    appendTo(list);

            this.pagination = new DmPagination(paginationDiv);
            this.pagination.setPageSize(3);
            this.pagination.onPageChange = this.getComments.bind(this);

            this.getComments();
        },

        getComments: function() {
            var comments = this.client.getComments(
                this.provider.id,
                this.pagination.getPagingParams().offset,
                this.pagination.getPagingParams().limit,
                this.repaintList.bind(this)
            );
        },

        repaintList: function(data) {
            var items = data.items;

            this.commentList.empty();
            if (0 != data.count) {
                for (var i = 0; i < items.length; i++) {
                    this.addCommentToTheList(this.commentList, items[i]);
                };
                this.pagination.setCount(data.count);
                this.pagination.renderPages();
            } else {
                this.commentList.html("<i>" + discount_search.comment_no_so_far + "</i>");
            }
        },

        addCommentToTheList: function(listElem, comment) {
            var commentDiv = $('<div />').
                    addClass('dm-comment-wrapper').
                    appendTo(listElem);

            var icon = $('<span />').
                    addClass('dm-user-icon').
                    appendTo(commentDiv);
            if (comment.consumer) {
                if (comment.consumer.photoUrl) {
                    icon.
                            css('background-image', 'url(' + comment.consumer.photoUrl + ')');
                } else {
                    icon.
                            addClass('fa').
                            addClass('fa-user');
                }
            } else {
                if (comment.type == 'PROVIDER') {
                    var logoUrl = this.provider.logoUrl;
                    $.each(this.provider.logo, function(ix, image) {
                        if (image.type == 'SMALL') {
                            logoUrl = image.url;
                            return false;
                        }
                    });
                    icon.
                            css('background-image', 'url(' + logoUrl + ')');
                } else {
                    icon.
                            addClass('dm-comment-reply-icon');
                }
            }

            var commentData = $('<div />').
                    addClass('dm-comment-data').
                    appendTo(commentDiv);

            if (comment.consumer) {
                $('<strong />').
                        addClass('dm-comment-username').
                        text(comment.consumer.username).
                        appendTo(commentData);
            } else {
                $('<strong />').
                        addClass('dm-comment-username').
                        text(comment.type == 'PROVIDER' ? this.localizer.localizedProviderValue(this.provider, 'name') : 'ISIC').
                        appendTo(commentData);
            }

            $('<span />').
                    addClass('dm-comment-date').
                    text(this.formatDate(new Date(comment.createdOn), this.localizer.message('date_format'))).
                    appendTo(commentData);

            $('<div />').
                    addClass('clearer').
                    appendTo(commentData);

            $('<p />').
                    addClass('dm-comment-text').
                    text(comment.text).
                    appendTo(commentData);

            $('<div />').
                    addClass('clearer').
                    appendTo(commentDiv);

            if (comment.replies.length != 0) {
                var repliesDiv = $('<div />').
                        addClass('dm-comment-replies-wrapper').
                        appendTo(commentDiv);
                for (var i = 0; i < comment.replies.length; i++) {
                    this.addCommentToTheList(
                        repliesDiv,
                        comment.replies[i]
                    );
                };
            }
        },

        formatDate: function(date, format) {
            return format.
                    replace('d', date.getDate() < 10 ? '0' + date.getDate() : date.getDate()).
                    replace('j', date.getDate()).
                    replace('m', (date.getMonth() + 1) < 10 ? '0' + (date.getMonth() + 1) : (date.getMonth() + 1)).
                    replace('n', date.getMonth() + 1).
                    replace('Y', date.getFullYear()).
                    replace('H', date.getHours() < 10 ? '0' + date.getHours() : date.getHours()).
                    replace('G', date.getHours()).
                    replace('i', date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()).
                    replace('s', date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds());
        }
    });

    window.DmRatingModule = function(providerId, initialValue, className, callback) {
        this.constructor.call(this, providerId, initialValue, className, callback);
    }
    $.extend(DmRatingModule.prototype, {

        providerId: null,
        value: null,
        className: null,
        callback: null,
        wrapper: null,

        constructor: function(providerId, initialValue, className, callback) {
            this.providerId = providerId;
            this.value = initialValue ? initialValue : 0;
            this.className = className;
            this.callback = callback ? callback : null;
        },

        render: function() {
            var whole, half;
            if (this.value) {
                whole = Math.floor(this.value);
                half = Math.pow(whole + 1, Math.round(this.value - whole));
            } else {
                whole = 0;
                half = 0;
            }

            var ratingDiv = $('<div />');
            this.wrapper = ratingDiv;
            if (this.className) {
                ratingDiv.addClass('dm-rating').addClass(this.className);
            }
            for (var i = 1; i < 6; i++) {
                var star = $('<span />').
                        addClass('dm-rating-star').
                        addClass('fa').
                        attr('data-value', i).
                        on(
                            'click',
                            this.rate.bind(this, i)
                        ).
                        appendTo(ratingDiv);
                if (i <= whole) {
                    star.
                            addClass('fa-star').
                            addClass('dm-active');
                } else if (i == half) {
                    star.
                            addClass('fa-star-half-full').
                            addClass('dm-active');
                } else {
                    star.
                            addClass('fa-star').
                            addClass('fa-inactive');
                }
            };
            var that = this;
            ratingDiv.find("span").mouseover(function() {
                that.highlight($(this).data('value'));
            }).mouseout(function() {
                that.highlight(that.value);
            });
            return ratingDiv;
        },

        getValue: function() {
            return this.value;
        },

        rate: function(rating, e) {
            this.value = rating;
            this.highlight(rating);
            if (this.callback !== null) {
                this.callback();
            }
        },

        highlight: function(value) {
            this.wrapper.
                    children('.dm-rating-star').
                    removeClass('dm-active').
                    removeClass('fa-star-half-full').
                    addClass('dm-inactive').
                    addClass('fa-star').
                    filter(
                        function() {
                            return $(this).data('value') <= value;
                        }
                    ).
                    removeClass('fa-star-half-full').
                    addClass('fa-star').
                    removeClass('dm-inactive').
                    addClass('dm-active');
            this.wrapper.
                    children('.dm-rating-star').
                    filter(
                        function() {
                            return value >= $(this).data('value') - 0.5 && value < $(this).data('value');
                        }
                    ).
                    removeClass('fa-star').
                    addClass('fa-star-half-full').
                    removeClass('dm-inactive').
                    addClass('dm-active');
        }

    });

})(jQuery);
